:- module( upsh, [upsh_make/0,upsh_make/1,upsh_version/1,upsh_version/2] ).

:- ensure_loaded( '../src/upsh_version' ).

upsh_make_defaults( Defs ) :- 
    current_prolog_flag( home, SwiHome ),
    % 1st place to look: ../../bin/swipl relative to flag home
    file_directory_name( SwiHome, SwiLib ),
    file_directory_name( SwiLib, BaseD ),
    directory_file_path( BaseD, bin, BinD ),
    Exec = upsh,
    ( exists_directory(BinD) ->
        Defs = [exec(Exec),bin_dir(BinD)]
        ;
        write( unable_to_locate_bin_dir_for_upsh__use_option(bin_dir) ), nl,
        Defs = [exec(Exec)]
    ).

upsh_make :-
    upsh_make( [] ).

upsh_make( ArgS ) :-
    ( is_list(ArgS) -> Args = ArgS; Args = [ArgS] ),
    upsh_make_defaults( Defs ),
    append( Args, Defs, Opts ),
    delete_local_exec,
    once( absolute_file_name( pack(upsh), UpshD ) ),
    working_directory( Old, UpshD ),
    Shell ='swipl -f upsh_create.pl -g upsh_create',
    debug( upsh, 'Shelling: ~w', Shell ),
    shell( Shell ),
    memberchk( exec(Exec), Opts ),
    memberchk( bin_dir(BinD), Opts ),
    directory_file_path( BinD, Exec, PathTo ),
    % rename_file( 'bin/upsh', PathTo ),
    atom_concat( 'mv bin/upsh ', PathTo, MvTo ),
    shell( MvTo ),
    debug( upsh, 'Moved to: ~p', PathTo ),
    working_directory( _, Old ),
    debug( upsh, 'Done', true ).

delete_local_exec :-
    Loc = 'bin/upsh',
    exists_file( Loc ),
    !,
    delete_file( Loc ).
delete_local_exec.
